/*
 * fancyqt4logfileview.h
 *
 *  Created on: 18.11.2011
 *      Author: wglas
 */

#ifndef FANCYQT4WIDGETS_LOGFILEVIEW_H_
#define FANCYQT4WIDGETS_LOGFILEVIEW_H_

#include <fancyqt4linerecord.h>
#include <QtGui/QAbstractScrollArea>

class QTextCodec;

namespace Fancy {
namespace Qt4 {
namespace Widgets {
namespace impl { class LogFileViewImpl; }

class FANCYQT4WIDGETS_API LogFileView : public QAbstractScrollArea {

	Q_OBJECT

	Q_PROPERTY(QString searchString READ getSearchString WRITE setSearchString)
	Q_PROPERTY(bool showLineNumbers READ isShowLineNumbers WRITE setShowLineNumbers)

	impl::LogFileViewImpl *d;

public:
	LogFileView(QWidget *parent);
	virtual ~LogFileView();

	Q_SLOT void openFile(const QString &path, QTextCodec * codec);
	Q_SLOT void setSearchString(const QString &search);
	Q_SLOT void scrollToBottom();
	Q_SLOT void setShowLineNumbers(bool b);

	Q_SLOT void copy();

	/**
	 * Request the watchdog thread to finish and return true, if the watchdog is not alive.
	 * If false is returned, the finished() signal is emitted when teh watchdog actually has finished.
	 */
	Q_SLOT bool requestClose();

	Q_SIGNAL void finished();

	Q_SIGNAL void copyAvailable(bool a);

	bool isCopyAvailable() const;
	QString getSelection() const;

	QString getSearchString() const;
	bool isShowLineNumbers() const;

protected:

    Q_SLOT void showLines(int firstLine, const Fancy::Qt4::Widgets::LineRecordList &lines);
    Q_SLOT void reload(int n, int lineHint);
    Q_SLOT void setTotalLines(int n);
    Q_SLOT void closeAccepted();
    Q_SLOT void showError(const QString &msg);

    void scrollContentsBy(int dx, int dy);
    void paintEvent(QPaintEvent *event);
    void closeEvent(QCloseEvent *event);
    void resizeEvent(QResizeEvent *event);
    void mouseMoveEvent(QMouseEvent* event);
    void mousePressEvent(QMouseEvent* event);
    void mouseReleaseEvent(QMouseEvent* event);
    void contextMenuEvent(QContextMenuEvent *event);

    int visibleWidth() const;
    int visibleHeight() const;
    int visibleLines() const;

    void reloadLines();
    void refreshLines();
};

} /* namespace Widgets */
} /* namespace Qt4 */
} /* namespace Fancy */
#endif /* FANCYQT4WIDGETS_LOGFILEVIEW_H_ */
